# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-10 10:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdyenIssuer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=200)),
                ('adyen_id', models.CharField(default='', max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name='AdyenNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('notification', models.TextField()),
                ('is_processed', models.BooleanField(default=False)),
                ('processed_at', models.DateTimeField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='AdyenPaymentOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=200)),
                ('adyen_name', models.CharField(default='', max_length=200)),
                ('guid', models.CharField(default='', max_length=36, verbose_name='GUID')),
                ('image', models.ImageField(null=True, upload_to='')),
                ('is_active', models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name='adyenissuer',
            name='payment_option',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djadyen.AdyenPaymentOption'),
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from djchoices import ChoiceItem, DjangoChoices


class Status(DjangoChoices):
    Authorised = ChoiceItem('authorised', _('Authorised'))  # The payment is completed.
    Cancel = ChoiceItem('cancel', _('Cancel'))  # The payment is canceld by the user.
    Created = ChoiceItem('created', _('Created'))  # The order is created.
    Error = ChoiceItem('error', _('Error'))  # The payment gave an error.
    Pending = ChoiceItem('pending', _('Pending'))  # The payment is not completed or rejected.
    Refused = ChoiceItem('refused', _('Refused'))  # The payment could not be done.



from django.test import override_settings
from django.urls import reverse

from django_webtest import WebTest
from webtest import AppError

from djadyen import settings
from djadyen.views import AdyenSigMixin

from .factories import IssuerFactory, OrderFactory, PaymentOptionsFactory


class MyRedirectViewTests(WebTest):
    def setUp(self):
        self.order = OrderFactory()
        self.url = reverse('redirect', kwargs={'reference': self.order.reference})

    def test_redirection_page(self):
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        response = form.submit()
        self.assertEqual(response.status_code, 302)

    def test_redirection_page_with_brand_code(self):
        self.order.payment_option = PaymentOptionsFactory()
        self.order.save()
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        response = form.submit()
        self.assertEqual(response.status_code, 302)

    def test_redirection_page_with_issuer(self):
        payment_option = PaymentOptionsFactory()
        self.order.payment_option = payment_option
        self.order.issuer = IssuerFactory(payment_option=payment_option)
        self.order.save()
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        response = form.submit()
        self.assertEqual(response.status_code, 302)

    @override_settings(ADYEN_ENABLED=True)
    def test_redirection_page_adyen_enabled(self):
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        with self.assertRaises(AppError):
            response = form.submit()

    @override_settings(ADYEN_ENABLED=True)
    def test_redirection_page_with_brand_code_adyen_enabled(self):
        self.order.payment_option = PaymentOptionsFactory()
        self.order.save()
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        with self.assertRaises(AppError):
            response = form.submit()

    @override_settings(ADYEN_ENABLED=True)
    def test_redirection_page_with_issuer_adyen_enabled(self):
        payment_option = PaymentOptionsFactory()
        self.order.payment_option = payment_option
        self.order.issuer = IssuerFactory(payment_option=payment_option)
        self.order.save()
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        with self.assertRaises(AppError):
            response = form.submit()

    @override_settings(ADYEN_ENABLED=True)
    def test_redirection_page_post_with_adyen_enabled(self):
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        with self.assertRaises(AppError):
            response = form.submit()


class My2RedirectViewTests(WebTest):
    def setUp(self):
        self.order = OrderFactory(email='')
        self.url = reverse('redirect2', kwargs={'reference': self.order.reference})

    def test_redirection_page(self):
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        response = form.submit()
        self.assertEqual(response.status_code, 302)

    def test_redirection_page_no_email(self):
        response = self.app.get(self.url)
        self.assertEqual(response.status_code, 200)

        form = response.forms['redirect-form']
        response = form.submit()
        self.assertEqual(response.status_code, 302)


class My3RedirectViewTests(WebTest):
    def setUp(self):
        self.order = OrderFactory(email='')
        self.url = reverse('redirect3', kwargs={'reference': self.order.reference})

    def test_redirection_page(self):
        self.order.email = ''
        self.order.save()
        with self.assertRaises(NotImplementedError):
            response = self.app.get(self.url)


class ConfirmationView(AdyenSigMixin, WebTest):
    def setUp(self):
        self.order = OrderFactory()
        self.url = reverse('confirm')

        self.params = {
            'merchantReturnData': 'test',
            'paymentMethod': 'ideal',
            'reason': 'test',
            'shopperLocale': 'test',
            'skinCode': 'test',
            'merchantReference': self.order.reference
        }

    def _get_csrf_token(self):
        url = reverse('redirect', kwargs={'reference': self.order.reference})
        response = self.app.get(url)
        return response.forms[0]['csrfmiddlewaretoken'].value

    def test_empty_get(self):
        with self.assertRaises(AppError):
            self.app.get(self.url)

    def test_wrong_signature_response(self):
        params = {
            'merchantReference': self.order.reference
        }
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_error_response(self):
        self.params['authResult'] = 'ERROR'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_cancelled_response(self):
        self.params['authResult'] = 'CANCELLED'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_refused_response(self):
        self.params['authResult'] = 'REFUSED'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_pending_response(self):
        self.params['authResult'] = 'PENDING'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_authorised_response(self):
        self.params['authResult'] = 'AUTHORISED'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_other_response(self):
        self.params['authResult'] = 'OTHER'
        params = self.sign_params(self.params)
        response = self.app.get(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_post(self):
        self.params['authResult'] = 'AUTHORISED'
        params = self.sign_params(self.params)
        params['csrfmiddlewaretoken'] = self._get_csrf_token()
        response = self.app.post(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')

    def test_psp_reference(self):
        self.params['pspReference'] = 'reference'
        params = self.sign_params(self.params)
        params['csrfmiddlewaretoken'] = self._get_csrf_token()
        response = self.app.post(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '\n    Success!\n\n')


class Confirmation2View(AdyenSigMixin, WebTest):
    def setUp(self):
        self.order = OrderFactory()
        self.url = reverse('confirm2')

        self.params = {
            'merchantReturnData': 'test',
            'paymentMethod': 'ideal',
            'reason': 'test',
            'shopperLocale': 'test',
            'skinCode': 'test',
            'merchantReference': self.order.reference,
            'pspReference': 'reference'
        }

    def test_empty_get(self):
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url)

    def test_wrong_signature_response(self):
        params = {
            'merchantReference': self.order.reference
        }
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_error_response(self):
        self.params['authResult'] = 'ERROR'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_cancelled_response(self):
        self.params['authResult'] = 'CANCELLED'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_refused_response(self):
        self.params['authResult'] = 'REFUSED'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_pending_response(self):
        self.params['authResult'] = 'PENDING'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_authorised_response(self):
        self.params['authResult'] = 'AUTHORISED'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)

    def test_other_response(self):
        self.params['authResult'] = 'OTHER'
        params = self.sign_params(self.params)
        with self.assertRaises(NotImplementedError):
            self.app.get(self.url, params=params)


class Confirmation3View(AdyenSigMixin, WebTest):
    def setUp(self):
        self.url = reverse('confirm3')

    def test_empty_get(self):
        self.app.get(self.url)



from django import forms

from djadyen import settings


class PaymentForm(forms.Form):
    skinCode = forms.CharField(initial=settings.ADYEN_SKIN_CODE)
    currencyCode = forms.CharField(initial=settings.ADYEN_CURRENCYCODE)
    merchantAccount = forms.CharField(initial=settings.ADYEN_MERCHANT_ACCOUNT)

    sessionValidity = forms.CharField()
    merchantReference = forms.CharField()
    paymentAmount = forms.CharField()

    merchantSig = forms.CharField()

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)

        initial = kwargs.get('initial')

        fixed_fields = [
            'skinCode', 'currencyCode', 'merchantAccount', 'sessionValidity', 'merchantReference',
            'paymentAmount', 'merchantSig'
        ]

        for key, value in initial.items():
            if key not in fixed_fields:
                self.fields[key] = forms.CharField(initial=value)



from datetime import datetime

from django.core.management.base import BaseCommand

import requests

from djadyen import settings

from ...models import AdyenIssuer, AdyenPaymentOption
from ...views import AdyenRequestMixin


class Command(AdyenRequestMixin, BaseCommand):
    help = "Sync the payment methods from adyen."

    def handle(self, *args, **options):
        params = {
            'merchantAccount': settings.ADYEN_MERCHANT_ACCOUNT,
            'skinCode': settings.ADYEN_SKIN_CODE,
            'merchantReference': 'payment_options',
            'currencyCode': settings.ADYEN_CURRENCYCODE,
            'paymentAmount': 199,
            'countryCode': 'NL',
            'sessionValidity': datetime.now().isoformat(),
        }
        params = self.sign_params(params)
        response = requests.post('{}hpp/directory.shtml'.format(settings.ADYEN_URL), params=params)

        payment_methods = response.json()['paymentMethods']
        for payment_method in payment_methods:
            payment_qs = AdyenPaymentOption.objects.filter(adyen_name=payment_method['brandCode'])
            if payment_qs.exists():
                payment = payment_qs.first()
                payment.name = payment_method.get('name')
                payment.save()
            else:
                payment = AdyenPaymentOption.objects.create(
                    name=payment_method.get('name'),
                    adyen_name=payment_method.get('brandCode')
                )

            if payment_method.get('issuers'):
                issuers = payment_method.get('issuers')
                for issuer in issuers:
                    issuer_qs = AdyenIssuer.objects.filter(adyen_id=issuer['issuerId'])
                    if issuer_qs.exists():
                        issuer_obj = issuer_qs.first()
                        issuer_obj.name = issuer.get('name')
                        issuer_obj.payment_option = payment
                        issuer_obj.save()
                    else:
                        issuer_obj = AdyenIssuer.objects.create(
                            name=issuer.get('name'),
                            adyen_id=issuer.get('issuerId'),
                            payment_option=payment
                        )



import base64
import binascii
import hashlib
import hmac
import logging
from collections import OrderedDict
from datetime import datetime, timedelta

from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.generic import FormView
from django.views.generic.detail import SingleObjectMixin

from djadyen import settings

from .forms import PaymentForm

try:
    from urllib.parse import urlencode
except Exception:
    from urllib import urlencode


logger = logging.getLogger('adyen')


class AdyenSigMixin(object):
    hmac_key = binascii.a2b_hex(settings.ADYEN_MERCHANT_SECRET)

    def escape_val(self, val):
        try:
            return val.replace('\\', '\\\\').replace(':', '\\:')
        except AttributeError:
            return str(val)

    def sign_params(self, parms):
        logger.debug("Params: %s", parms)

        parms = OrderedDict(sorted(parms.items(), key=lambda t: t[0]))
        logger.debug("Ordered Params: %s", parms)

        signing_string = ':'.join(map(self.escape_val, list(parms.keys()) + list(parms.values())))
        logger.debug("Signing Params: %s", signing_string)

        hmac_string = hmac.new(self.hmac_key, signing_string.encode('utf-8'), hashlib.sha256)
        logger.debug("HMAC: %s", hmac_string)

        parms['merchantSig'] = base64.b64encode(hmac_string.digest()).decode("utf-8")
        logger.debug("merchantSig: %s", parms['merchantSig'])

        return parms


class AdyenRequestMixin(AdyenSigMixin):
    def adyen_url(self, brand_code=None, issuer_id=None):
        page = 'hpp/select.shtml'
        if brand_code:
            page = 'hpp/details.shtml'
        if issuer_id:
            page = 'hpp/skipDetails.shtml'
        url = '{}{}'.format(settings.ADYEN_URL, page)

        logger.debug("Adyen Url: %s", url)
        return url


class AdyenRedirectView(AdyenRequestMixin, SingleObjectMixin, FormView):
    template_name = 'adyen/form.html'
    form_class = PaymentForm
    slug_field = 'reference'
    slug_url_kwarg = 'reference'

    default_params = {
        'skinCode': settings.ADYEN_SKIN_CODE,
        'currencyCode': settings.ADYEN_CURRENCYCODE,
        'merchantAccount': settings.ADYEN_MERCHANT_ACCOUNT,
    }

    def get_signed_order_params(self, order, **kwargs):
        params = self.get_default_params(
            merchantReference=order.reference,
            paymentAmount=order.get_price_in_cents(),
            resURL='{}{}'.format(settings.ADYEN_HOST_URL, self.get_next_url())
        )
        if order.email:
            params['shopperEmail'] = order.email
        if order.payment_option:
            params['brandCode'] = order.payment_option.adyen_name
        if order.issuer:
            params['issuerId'] = order.issuer.adyen_id

        params.update(**kwargs)

        return self.sign_params(params)

    def get_default_params(self, **kwargs):
        params = self.default_params.copy()
        valid = datetime.now() + timedelta(minutes=settings.ADYEN_SESSION_MINUTES)
        params.update(
            sessionValidity=valid.isoformat(),
        )
        params.update(**kwargs)
        return params

    def get_context_data(self, **kwargs):
        context = super(AdyenRedirectView, self).get_context_data(**kwargs)
        if settings.ADYEN_ENABLED:
            brand_code = None
            issuer_id = None
            if hasattr(self.object, "payment_option") and self.object.payment_option:
                brand_code = self.object.payment_option
            if hasattr(self.object, "issuer") and self.object.issuer:
                issuer_id = self.object.issuer.id
            context.update({
                'adyen_url': self.adyen_url(brand_code=brand_code, issuer_id=issuer_id),
                'add_csrf': False
            })
        else:
            context.update({
                'adyen_url': self.get_current_url(),
                'add_csrf': True
            })
        return context

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        if self.can_skip_payment():
            return self.skip_payment()

        response = super(AdyenRedirectView, self).get(request, *args, **kwargs)
        self.perform_action()
        return response

    def can_skip_payment(self):
        """
        A function to determain if we should skip the payment redirect
        """
        return False

    def skip_payment(self):
        raise NotImplementedError("You can't skip payments yet")

    def post(self, request, *args, **kwargs):
        """
        Normaly this post will go to Adyen. But for local development and intergrated testing this
        function can be used so no user action is required to test the handling.
        """
        if settings.ADYEN_ENABLED:
            raise Http404("Cannot handle post when adyen is enabled")

        order = self.get_object()
        params = {
            "authResult": settings.ADYEN_NEXT_STATUS,
            "pspReference": "Local redirect",
            "merchantReference": order.reference,
            "skinCode": settings.ADYEN_SKIN_CODE,
            "paymentMethod": "ideal",
            "shopperLocale": "NL",
        }

        signed_params = self.sign_params(params)
        return HttpResponseRedirect("{}?{}".format(self.get_next_url(), urlencode(signed_params)))

    def perform_action(self):
        pass

    def get_next_url(self):
        pass

    def get_current_url(self):
        return "."


class AdyenResponseMixin(AdyenSigMixin):
    auto_fetch = True

    def post(self, request, *args, **kwargs):
        request.GET = request.POST
        return self.get(request, *args, **kwargs)

    def done(self):
        if self.auto_fetch:
            self.order.save()

        return self.render_to_response(self.get_context_data())

    def get(self, request, *args, **kwargs):
        self.auth_result = request.GET.get('authResult')
        self.psp_reference = request.GET.get('pspReference')
        self.merchant_reference = request.GET.get('merchantReference')
        self.skin_code = request.GET.get('skinCode')
        self.payment_method = request.GET.get('paymentMethod')
        self.shopper_locale = request.GET.get('shopperLocale')
        self.merchant_return_data = request.GET.get('merchantReturnData')
        self.reason = request.GET.get('reason')

        self.merchant_sig = request.GET.get('merchantSig')

        params = {
            'authResult': self.auth_result,
            'merchantReference': self.merchant_reference,
            'skinCode': self.skin_code,
            'paymentMethod': self.payment_method,
            'shopperLocale': self.shopper_locale,
        }

        if self.psp_reference:
            params['pspReference'] = self.psp_reference
        if self.merchant_return_data:
            params['merchantReturnData'] = self.merchant_return_data
        if self.reason:
            params['reason'] = self.reason

        sig = self.sign_params(params)

        if self.auto_fetch:
            try:
                self.order = get_object_or_404(self.model, reference=self.merchant_reference)
            except Exception:
                raise Http404

        self.handle_default()
        logger.info(
            'Order ref: %s | Received Adyen auth result: %s',
            self.merchant_reference,
            self.auth_result
        )

        if sig['merchantSig'] != self.merchant_sig and settings.ADYEN_ENABLED:
            logger.debug('Order ref: %s | MerchangeSig not correct', self.merchant_reference)
            logger.debug(
                'Order ref: %s | Our Msig: %s | Adyen Msig: %s',
                self.merchant_reference,
                sig['merchantSig'],
                self.merchant_sig
            )
            return self.handle_error()

        if self.auth_result == 'ERROR':
            return self.handle_error()

        if self.auth_result == 'CANCELLED':
            return self.handle_canceled()

        if self.auth_result == 'REFUSED':
            return self.handle_refused()

        if self.auth_result == 'PENDING':
            return self.handle_pending()

        if self.auth_result == 'AUTHORISED':
            return self.handle_authorised()

        logger.error("Please implement the following authResult: %s", self.auth_result)
        return self.handle_pending()

    def handle_authorised(self):
        raise NotImplementedError()

    def handle_pending(self):
        raise NotImplementedError()

    def handle_refused(self):
        raise NotImplementedError()

    def handle_error(self):
        raise NotImplementedError()

    def handle_canceled(self):
        raise NotImplementedError()

    def handle_default(self):
        return



import json

from django.core.management import call_command
from django.test.testcases import TestCase

import requests_mock

from djadyen.models import AdyenIssuer, AdyenPaymentOption


def json_response(request, context):
    return json.dumps({
        'paymentMethods': [
            {'brandCode': 'mc', 'name': 'MasterCard'},
            {'brandCode': 'visa', 'name': 'VISA'},
            {'brandCode': 'ideal', 'issuers': [
                {'issuerId': '1121', 'name': 'Test Issuer'},
                {'issuerId': '1154', 'name': 'Test Issuer 5'},
                {'issuerId': '1153', 'name': 'Test Issuer 4'},
                {'issuerId': '1152', 'name': 'Test Issuer 3'},
                {'issuerId': '1151', 'name': 'Test Issuer 2'},
                {'issuerId': '1162', 'name': 'Test Issuer Cancelled'},
                {'issuerId': '1161', 'name': 'Test Issuer Pending'},
                {'issuerId': '1160', 'name': 'Test Issuer Refused'},
                {'issuerId': '1159', 'name': 'Test Issuer 10'},
                {'issuerId': '1158', 'name': 'Test Issuer 9'},
                {'issuerId': '1157', 'name': 'Test Issuer 8'},
                {'issuerId': '1156', 'name': 'Test Issuer 7'},
                {'issuerId': '1155', 'name': 'Test Issuer 6'}
            ], 'name': 'iDEAL'}
        ]}, ensure_ascii=False).encode('gbk')


class SyncPaymentMethods(TestCase):
    def test_on_empty_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

    def test_on_existing_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

        # call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

    @requests_mock.mock()
    def test_on_empty_database_mock(self, mock):
        mock.post(
            'https://test.adyen.com/hpp/directory.shtml',
            [
                {"content": json_response, "status_code": 200},
            ],
        )

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)

    @requests_mock.mock()
    def test_on_existing_database_mock(self, mock):
        mock.post(
            'https://test.adyen.com/hpp/directory.shtml',
            [
                {"content": json_response, "status_code": 200},
                {"content": json_response, "status_code": 200},
            ],
        )
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)



try:
    from django.urls import path
except:
    from django.conf.urls import url as path

from .views import NotificationView

app_name = 'adyen-notifications'
urlpatterns = [
    path(r'^$', NotificationView.as_view(), name="notification"),
]



from django.urls import reverse

from django_webtest import WebTest

from djadyen.models import AdyenNotification


class RedirectViewTests(WebTest):
    def setUp(self):
        self.url = reverse('adyen-notifications:notification')

    def test_post_notification_empty(self):
        response = self.app.post(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '[accepted]')
        self.assertEqual(AdyenNotification.objects.count(), 0)

    def test_post_notification(self):
        params = {
            'eventDate': '2018-01-01T01:02:01.111Z',
            'reason': '58747:1111:6/2018',
            'additionalData.cardSummary': ' 1111',
            'originalReference': '',
            'merchantReference': 'YourMerchantReference1',
            'additionalData.expiryDate': '8/2018',
            'currency': 'EUR',
            'pspReference': '8888777766665555',
            'additionalData.authCode': '58747',
            'merchantAccountCode': 'TestMerchant',
            'eventCode': 'AUTHORISATION',
            'value': '500',
            'operations': 'CANCEL,CAPTURE,REFUND',
            'success': 'true',
            'paymentMethod': 'visa',
            'live': 'false',
        }
        response = self.app.post(self.url, params=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, '[accepted]')
        self.assertEqual(AdyenNotification.objects.count(), 1)



from django.utils import timezone
from django.urls import reverse
from django.views.generic import TemplateView
from django.utils import timezone

from djadyen.choices import Status
from djadyen.views import AdyenRedirectView, AdyenResponseMixin

from .models import Order


class MyAdyenRequestView(AdyenRedirectView):
    model = Order

    def get_form_kwargs(self):
        order = self.get_object()
        params = self.get_signed_order_params(order)

        kwargs = super(MyAdyenRequestView, self).get_form_kwargs()
        kwargs.update({'initial': params})
        return kwargs

    def get_next_url(self):  # This is to populate the resURL
        return reverse('confirm')


class My2AdyenRequestView(AdyenRedirectView):
    model = Order

    def get_form_kwargs(self):
        params = self.get_default_params(
            shipBeforeDate=timezone.now(), merchantReturnData='returnData')

        kwargs = super(My2AdyenRequestView, self).get_form_kwargs()
        kwargs.update({'initial': params})
        return kwargs


class My3AdyenRequestView(AdyenRedirectView):
    model = Order

    def get_form_kwargs(self):
        params = self.get_default_params(
            shipBeforeDate=timezone.now(), merchantReturnData='returnData')

        kwargs = super(My3AdyenRequestView, self).get_form_kwargs()
        kwargs.update({'initial': params})
        return kwargs

    def can_skip_payment(self):
        return True


class ConfirmationView(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    model = Order

    def handle_authorised(self):
        self.order.status = Status.Authorised
        return self.done()

    def handle_pending(self):
        self.order.status = Status.Pending
        return self.done()

    def handle_refused(self):
        self.order.status = Status.Refused
        return self.done()

    def handle_error(self):
        self.order.status = Status.Error
        return self.done()

    def handle_canceled(self):
        self.order.status = Status.Cancel
        return self.done()

    def handle_default(self):
        if self.psp_reference:
            self.order.psp_reference = self.psp_reference


class Confirmation2View(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    auto_fetch = False


class Confirmation3View(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    auto_fetch = False

    def handle_authorised(self):
        return self.done()

    def handle_pending(self):
        return self.done()

    def handle_refused(self):
        return self.done()

    def handle_error(self):
        return self.done()

    def handle_canceled(self):
        return self.done()



import sys

from django.conf import settings


class _Settings(object):
    @property
    def ADYEN_HOST_URL(self):
        return getattr(settings, 'ADYEN_HOST_URL')

    @property
    def ADYEN_MERCHANT_ACCOUNT(self):
        return getattr(settings, 'ADYEN_MERCHANT_ACCOUNT')

    @property
    def ADYEN_MERCHANT_SECRET(self):
        return getattr(settings, 'ADYEN_MERCHANT_SECRET')

    @property
    def ADYEN_SKIN_CODE(self):
        return getattr(settings, 'ADYEN_SKIN_CODE')

    @property
    def ADYEN_CURRENCYCODE(self):
        return getattr(settings, 'ADYEN_CURRENCYCODE', 'EUR')

    @property
    def ADYEN_ENABLED(self):
        return getattr(settings, 'ADYEN_ENABLED', True)

    @property
    def ADYEN_NEXT_STATUS(self):
        return getattr(settings, 'ADYEN_NEXT_STATUS', 'AUTHORISED')

    @property
    def ADYEN_REFETCH_OLD_STATUS(self):
        return getattr(settings, 'ADYEN_REFETCH_OLD_STATUS', False)

    @property
    def ADYEN_SESSION_MINUTES(self):
        return getattr(settings, 'ADYEN_SESSION_MINUTES', 10)

    @property
    def ADYEN_URL(self):
        return getattr(settings, 'ADYEN_URL', 'https://test.adyen.com/')

    def __getattr__(self, name):
        return globals()[name]


# other parts of itun that you WANT to code in
# module-ish ways
sys.modules[__name__] = _Settings()



from django.test import TestCase, override_settings

from djadyen import settings
from djadyen.choices import Status

from .factories import (
    IssuerFactory, NotificationFactory, OrderFactory, PaymentOptionsFactory
)


class AdyenNofiticationTests(TestCase):
    def test_str(self):
        notification = NotificationFactory()
        self.assertEqual(notification.__str__(), notification.created_at.strftime('%c'))


class AdyenOrderTests(TestCase):
    def test_str(self):
        with self.assertNumQueries(1):
            order = OrderFactory()
            self.assertEqual(order.__str__(), '{}'.format(order.reference))

    @override_settings(ADYEN_REFETCH_OLD_STATUS=True)
    def test_refetch_old_status(self):
        with self.assertNumQueries(3):
            order = OrderFactory()
            self.assertEqual(order.__str__(), '{}'.format(order.reference))

            order.status = Status.Authorised
            order.save()
            self.assertEqual(order.__str__(), '{}'.format(order.reference))

    def test_can_not_overwrite_authorised_status(self):
        with self.assertNumQueries(2):
            order = OrderFactory(status=Status.Authorised)
            self.assertEqual(order.status, 'authorised')

            order.status = Status.Cancel
            order.save()
            self.assertEqual(order.status, 'authorised')


class AdyenIssuerTests(TestCase):
    def test_str(self):
        issuer = IssuerFactory()
        self.assertEqual(issuer.__str__(), issuer.name)


class AdyenPaymentOptionTests(TestCase):
    def test_str(self):
        payment_option = PaymentOptionsFactory()
        self.assertEqual(payment_option.__str__(), payment_option.name)
